{% load watson %}
{% if search_results %}
    <ul class="search-results">
        {% for search_result in search_results %}
            <li>
                {% search_result_item search_result %}
            </li>
        {% endfor %}
    </ul>
{% endif %}